@@ -2,8 +2,10 @@ import os
 import re
 import shutil
 from abc import ABC, abstractmethod
+from io import BytesIO
 
 import pytz
+import pandas as pd
 import xlrd
 from loguru import logger
 from minio import Minio
@@ -11,6 +13,8 @@ from pathlib import Path
 from datetime import datetime
 
 # prefix components:
+from minio.select import SelectRequest, CSVInputSerialization, CSVOutputSerialization
+
 space = '    '
 branch = '│   '
 # pointers:
@@ -197,6 +201,30 @@ class MinioFS(FileSystem):
             data.append(data_stream)
         return data
 
+    def query_csv(self, filename: str, sql: str):
+        bucket, filepath = self._process_path(filename)
+        data = self.__client__.select_object_content(bucket,
+                                                     filepath,
+                                                     SelectRequest(sql,
+                                                                   CSVInputSerialization(file_header_info="USE"),
+                                                                   CSVOutputSerialization(),
+                                                                   request_progress=True)
+                                                     )
+
+        headers = self.__client__.select_object_content(bucket,
+                                                        filepath,
+                                                        SelectRequest("select * from s3object limit 1",
+                                                                      CSVInputSerialization(file_header_info="NONE"),
+                                                                      CSVOutputSerialization(),
+                                                                      request_progress=True)
+                                                        )
+        bytearray = b""
+        for line in headers.stream():
+            bytearray = bytearray + line
+        for line in data.stream():
+            bytearray = bytearray + line
+        return pd.read_csv(BytesIO(bytearray))
+
     def rm(self, paths):
         """
         :param paths:
